@@ -307,23 +307,26 @@ class Solr(object):
             http = Http(timeout=self.timeout)
             url = self.base_url + path
 
+            self.log.debug("Starting request to '%s' (%s) with body '%s'...",
+                           url, method, str(body)[:10])
+            start_time = time.time()
+
             try:
-                start_time = time.time()
-                self.log.debug("Starting request to '%s' (%s) with body '%s'...",
-                               url, method, str(body)[:10])
                 headers, response = http.request(url, method=method, body=body, headers=headers)
-                end_time = time.time()
-                self.log.info("Finished '%s' (%s) with body '%s' in %0.3f seconds.",
-                              url, method, str(body)[:10], end_time - start_time)
-            except AttributeError:
+            except (IOError, AttributeError):
                 error_message = "Failed to connect to server at '%s'. Are you sure '%s' is correct? Checking it in a browser might help..."
                 params = (url, self.base_url)
-                self.log.error(error_message, *params)
+                self.log.error(error_message, *params, exc_info=True)
                 raise SolrError(error_message % params)
 
+            end_time = time.time()
+            self.log.info("Finished '%s' (%s) with body '%s' in %0.3f seconds.",
+                          url, method, str(body)[:10], end_time - start_time)
+
             if int(headers['status']) != 200:
                 error_message = self._extract_error(headers, response)
-                self.log.error(error_message)
+                self.log.error(error_message, extra={'data': {'headers': headers,
+                                                              'response': response}})
                 raise SolrError(error_message)
 
             return response
@@ -339,15 +342,26 @@ class Solr(object):
             start_time = time.time()
             self.log.debug("Starting request to '%s:%s/%s' (%s) with body '%s'...",
                            self.host, self.port, path, method, str(body)[:10])
-            conn.request(method, path, body, headers)
+
+            try:
+                conn.request(method, path, body, headers)
+            except IOError:
+                error_message = "Failed to connect to server at '%s'. Are you sure '%s' is correct? Checking it in a browser might help..."
+                params = (path, self.base_url)
+                self.log.error(error_message, *params, exc_info=True)
+                raise SolrError(error_message % params)
+
             response = conn.getresponse()
             end_time = time.time()
             self.log.info("Finished '%s:%s/%s' (%s) with body '%s' in %0.3f seconds.",
                           self.host, self.port, path, method, str(body)[:10], end_time - start_time)
 
             if response.status != 200:
-                error_message = self._extract_error(dict(response.getheaders()), response.read())
-                self.log.error(error_message)
+                resp_headers = dict(response.getheaders())
+                resp_body = response.read()
+                error_message = self._extract_error(resp_headers, resp_body)
+                self.log.error(error_message, extra={'data': {'headers': resp_headers,
+                                                              'response': resp_body}})
                 raise SolrError(error_message)
 
             return response.read()
@@ -435,6 +449,7 @@ class Solr(object):
         # identify the responding server
         server_type = None
         server_string = headers.get('server', '')
+        content_type = headers.get('content-type', '')
 
         if server_string and 'jetty' in server_string.lower():
             server_type = 'jetty'
@@ -445,11 +460,38 @@ class Solr(object):
             from BeautifulSoup import BeautifulSoup
             server_type = 'tomcat'
 
+        # Solr 4 handle errors by itself
+        if content_type.startswith('application/json'):
+            server_type = 'solr4_json'
+
+        if content_type.startswith('application/xml'):
+            server_type = 'solr4_xml'
+
         reason = None
         full_html = ''
         dom_tree = None
 
-        if server_type == 'tomcat':
+        if server_type == 'solr4_json':
+            try:
+                data = json.loads(response)
+                error = data['error']
+                reason = error.get('msg') or error.get('trace')
+            except (ValueError, KeyError):
+                pass
+        elif server_type == 'solr4_xml':
+            try:
+                tree = ET.fromstring(response)
+                lst_nodes = tree.findall('lst')
+
+                for lst_node in lst_nodes:
+                    if lst_node.get('name') == 'error':
+                        msg_node = lst_node.find('str')
+                        if msg_node is not None and msg_node.get('name') in ('msg', 'trace'):
+                            reason = msg_node.text
+                            break
+            except SyntaxError, e:
+                pass
+        elif server_type == 'tomcat':
             # Tomcat doesn't produce a valid XML response
             soup = BeautifulSoup(response)
             body_node = soup.find('body')
@@ -460,9 +502,6 @@ class Solr(object):
 
                 if len(children) >= 2 and 'message' in children[0].renderContents().lower():
                     reason = children[1].renderContents()
-
-            if reason is None:
-                full_html = soup.prettify()
         else:
             # Let's assume others do produce a valid XML response
             try:
@@ -472,6 +511,8 @@ class Solr(object):
                 # html page might be different for every server
                 if server_type == 'jetty':
                     reason_node = dom_tree.find('body/pre')
+                else:
+                    reason_node = dom_tree.find('head/title')
 
                 if reason_node is not None:
                     reason = reason_node.text
@@ -479,7 +520,10 @@ class Solr(object):
                 if reason is None:
                     full_html = ET.tostring(dom_tree)
             except SyntaxError, e:
-                full_html = "%s" % response
+                pass
+
+        if reason is None:
+            full_html = "%s" % response
 
         full_html = full_html.replace('\n', '')
         full_html = full_html.replace('\r', '')
@@ -796,14 +840,14 @@ class Solr(object):
                                       "".join(body_generator), headers)
         except (IOError, SolrError),  e:
             self.log.error("Failed to extract document metadata: %s", e,
-                           exc_info=e)
+                           exc_info=True)
             raise
 
         try:
             data = json.loads(resp)
         except ValueError, e:
             self.log.error("Failed to load JSON response: %s", e,
-                           exc_info=e)
+                           exc_info=True)
             raise
 
         data['contents'] = data.pop(file_obj.name, None)
